在jxWebUI中，用户可以同时打开多个界面，每个界面在tabs占据一个tab。在tabs中占据了一个tab的page，我们称为主page。

但很多时候，我们在主page中进行操作时，可能需要打开另一个界面做一些事，但这些事却是依托当前主page的，如制作销售订单时，根据客户的信用、购买历史、特批等核算出不同的折扣。

而我们之前反复说过，一个tab对应一个capaInstance，而一个capaInstance则独立管理用户在page中的输入输出，也就是说：**两个tab之间的数据是严格隔离开的**。

所以，我们必须有一种依托当前tab向用户显示辅助page的手段，在jxWebUI中，就是对话框。

对话框是模态窗口，所以同时只能打开一个对话框。

在一个主page中打开另一个page是非常常见的推动工作流的动作。那么何时打开另一个主page【显示到另一个tab中】、何时打开对话框呢？

另一个page信息量比较大、操作比较多、业务逻辑上是当前工作的下一步或独立环节，则应当以主page的方式显示到一个新的tab中。

另一个page信息量不大、操作就一个【点击对话框确认按钮执行该操作】、业务逻辑上依托于当前工作，属于当前工作的一个组成部分，则应该以对话框的形式弹窗快速完成。

因此一个tab只能有一个主page，同时还可以有多个附属的page，也就是对话框。

更本质一点，一个tab就对应一个capaInstance。主page和对话框共享这一个capaInstance，所以主page和各对话框中的输入数据是共享的。

而两个不同的tab中的page，哪怕都是打开的同一个capa中的page，但实际上它们对应的却是不同的capaInstance，所以它们的数据是相互隔离的。

## 定义格式

对话框也是page，和主page的区别主要有两点：

1、需要设置page的okBtn属性

2、在定义打开对话框page的入口时，需要指定dispInDialog属性为True

3、对话框的宽度应为600

<font color=red size=3>注：</font>打开对话框的入口一般是主page中的按钮或工具栏中的工具条

示例：

	@capa.web
	def dialog_test(page):
	    t = page.table().width(600).title('对话框111')
	    r = t.row()
	    r.text().text('对话框文本框1').width(200)
	    r.input().bind('test_input').width(200).placeholder('dialog_test对话框输出')
	
	    page.okBtn().motion('cmd').demand('test111')

显示效果：

![test_web](http://115.29.52.95:10018/images/dhk_1.png)

显示该对话框的入口示例：

    #查询表操作列中显示一个对话框，和一般的操作列工具条的参数主要就是如何显示的区别：
    #显示到对话框：dispInDialog(True)，即在同一个capaInstance下显示到对话框中
    #显示到主page：primary(True)、ignoreCapaid(True)，即忽略capaid显示到tab中
    t.col('op1').head('操作').width(100).a().text('测试对话框').motion('disp').demand('dialog_test').require(['dev_name']).dispInDialog(True)

可以看出：

1、对话框就是在通常的容器表定义之上多了一个okBtn的定义

2、okBtn就等价于一个工具条【a】，点击确认按钮，就相当于点击了这个工具条

3、对话框的宽度应该设为600，否则会和底层的模态窗口不匹配

所以，对话框的定义包括三个部分：

- 本对话框的主体容器表的定义
- 工具条【确认按钮点击后需要执行的cmd事件】的定义

定义好对话框后，需要为其指定相应的入口。添加到工具栏中的定义是：

	capa.btn_list_add('test_web', 'disp', 'dialog_test', dispInDialog=True, text='对话框')

就会在test_web这个主page中添加一个工具栏中的工具条：

![test_web](http://115.29.52.95:10018/images/web_1.png)